#include "TecModule.h"
#include "TecParameters.h"
#include <tec/TecChimeraInterface.h>
#include <tec/TecController.h>

/**********************************************************************************************************************/

TecModule::TecModule(EntityOwner* owner, const std::string& name, const std::string& description,
    ctk::HierarchyModifier hierarchyModifier, const std::unordered_set<std::string>& tags)
: ctk::ApplicationModule(owner, name, description, hierarchyModifier, tags) {
  // example how to fill the map
  readIntMap.emplace(
      TecFrame::TecParameter::Invalid, ChimeraTK::ScalarOutput<int>{this, "SOME_PV.RD", "unit", "description"});

  // Note: The maps writeGroupMap, writeGroupToFlashMap, readGroupMap need to be filled in the mainLoop(). The
  // corresponding ScalarPushInputs need to be created here and placed onto the groupCommandMap.

  // example how to fill a group command
  groupCommandMap.emplace("MY_READ_GROUP",
      ChimeraTK::ScalarPushInput<int>{this, "MY_READ_GROUP.WR", "", "Trigger read of group MY_READ_GROUP"});
}

/**********************************************************************************************************************/

void TecModule::mainLoop() {
  std::cout << "Processing TecModule::mainLoop()" << std::endl;

  auto tecIface = std::make_shared<TecChimeraInterface>(
      triggerCommands, responseStatus, commandMemoryRaw_wr, commandMemoryRaw_rd, true);
  TecController tec{tecIface};
  tec.addFrame(0, TecFrame::TecParameter::ExpFanControlEnable, TecFrame::TecFrameType::TecParameterValueSet, 1, 42);
  tec.addFrame(0, TecFrame::TecParameter::DeviceStatus, TecFrame::TecFrameType::TecParameterValueRead, 1);
  std::cout << "constructAllFrames:" << std::endl;
  tec.constructAllFrames();
  for(uint8_t i = 0; i < tec.getFrameCount(); i++) {
    tec.getFrameAt(i)->print();
  }
  std::cout << "copyCommandsToHardware:" << std::endl;
  tec.copyCommandsToHardware();
  std::cout << "triggerHardwareCommands:" << std::endl;
  tec.triggerHardwareCommands();
  std::cout << "readHardwareResponses:" << std::endl;
  tec.readHardwareResponses();
  std::cout << "copyResponsesFromHardware:" << std::endl;
  tec.copyResponsesFromHardware();
  std::cout << "parseAllFrames:" << std::endl;
  tec.parseAllFrames();
  for(uint8_t i = 0; i < tec.getFrameCount(); i++) {
    tec.getFrameAt(i)->print();
  }
  std::cout << "done." << std::endl;
  
  // Here the group maps need to be filled by iterating the descriptor list and obtaining the corresponding inputs from
  // the groupCommandMap
  readGroupMap.emplace(groupCommandMap.at("MY_READ_GROUP").getId(),
      std::list<TecFrame::TecParameter>{TecFrame::TecParameter::MonSinkTemperature});

  auto group = readAnyGroup();
  while(true) {
    auto id = group.readAny();
    if(id == periodicRead.trigger.getId()) {
      readPeriodic();
    }
    else if(readGroupMap.find(id) != readGroupMap.end()) {
      readGroup(readGroupMap.at(id));
    }
    else if(writeGroupMap.find(id) != writeGroupMap.end()) {
      writeGroup(writeGroupMap.at(id));
    }
    else if(writeGroupToFlashMap.find(id) != writeGroupToFlashMap.end()) {
      writeGroupToFlash(writeGroupToFlashMap.at(id));
    }
    else {
      std::cout << "Unexpected input (id = " << id << ") has been written, don't know what to do!" << std::endl;
    }
  }
}

/**********************************************************************************************************************/

void TecModule::readPeriodic() {
  std::cout << "readPeroidoc()" << std::endl;
}

/**********************************************************************************************************************/

void TecModule::readGroup(std::list<TecFrame::TecParameter>& group) {
  std::cout << "readGroup()" << std::endl;
  for(auto& param : group) std::cout << "Read param: " << param << std::endl;
}

/**********************************************************************************************************************/

void TecModule::writeGroup(std::list<TecFrame::TecParameter>& group) {
  std::cout << "writeGroup()" << std::endl;
  for(auto& param : group) std::cout << "Rrite param: " << param << std::endl;
}

/**********************************************************************************************************************/

void TecModule::writeGroupToFlash(std::list<TecFrame::TecParameter>& group) {
  std::cout << "writeGroupToFlash()" << std::endl;
  for(auto& param : group) std::cout << "Write param to flash: " << param << std::endl;
}

/**********************************************************************************************************************/
